use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Blanco
Byrne
CPAN
Chase
Christopher
Clendenan
Constants
Dave
David
Deserializer
Foos
Fred
Gordon
Gross
Martin
Mckay
Moyer
Nicholas
Reese
SOQL
SOSL
Salesforce
Serializer
Shimizu
Simple
Stanton
Steinbrunner
Stubblebine
WWW
Whitener
byrne
capoeirab
checkAsyncStatus
checkRetrieveStatus
cmckay
complexTypes
convertLead
creddi
dclendenan
describeGlobal
describeLayout
describeMetadata
describeSObject
describeSObjects
describeTabs
dreamForce
dsteinbrunner
fred
getDeleted
getErrorDetails
getServerTimestamp
getUpdated
getUserInfo
github
gmail
gordons
lib
majordojo
martin
nfoos
phred
queryAll
queryLocator
queryMore
redcharlie
redhotpenguin
resetPassword
retrieveMetadata
salesforce
searchString
serverUrl
serverurl
setPassword
upsert
userId
